module Test.TvSetFeatureSpec where

  import Control.Monad.Trampoline (runTrampoline)
  import Control.Monad.Trans

  import Data.Array.Unsafe (head)

  import Test.StrongCheck (Arbitrary, arbitrary, quickCheck)
  import Test.StrongCheck.Gen (sample)
  import Test.FeatureSpec

  newtype TvSet = TvSet {on :: Boolean}

  instance eqTvSet :: Eq TvSet where
    (==) (TvSet tv) (TvSet tv') = tv.on == tv'.on

    (/=) tv tv' = not $ tv == tv'

  instance arbitraryTvSet :: Arbitrary TvSet where
    arbitrary = do
      b <- arbitrary
      pure $ TvSet {on: b}

  tvSet :: TvSet
  tvSet = TvSet {on: false}

  isOn :: TvSet -> Boolean
  isOn (TvSet tv) = tv.on

  pressPowerButton :: TvSet -> TvSet
  pressPowerButton (TvSet tv) = TvSet tv{on = not tv.on}

  pressTwice :: TvSet -> Boolean
  pressTwice tv = tv == (pressPowerButton $ pressPowerButton tv)

  tvSetFeatureSpec = spec "TvSetSpec" do
    info "As a TV set owner"
    info "I want to be able to turn the TV on and off"
    info "So I can watch TV when I want"
    info "And save energy when I'm not watching TV"

    feature "TV power button" do
      scenario "User presses power button when TV is off" do

        given "a TV set that is switched off"
        let tv = tvSet
        assert' (not $ isOn tv) "tv is off"

        when "the power button is pressed"
        let tv' = pressPowerButton tv

        it "should switch on"
        assert' (isOn tv') "tv should be on"

      scenario "User presses power button when TV is on" do

        given "a TV set that is switched on"
        let tv = tvSet
        let tv' = pressPowerButton tv
        assert' (isOn tv') "tv starts on"

        when "the power button is pressed"
        let tv'' = pressPowerButton tv'

        it "should switch off"
        assert' (not $ isOn tv'') "tv should be off"

      scenario "This one should be pending" do
        pending

      ignore "This one should be ignored" do

        given "a TV set that is switched off"
        let tv = tvSet
        assert' (not $ isOn tv) "tv is off"

        when "the power button is pressed"
        let tv' = pressPowerButton tv

        it "should switch on"
        assert' (isOn tv') "tv should be on"

      scenario "QuickCheck should work" do
        given "some arbitrary tv set"
        -- tv <- lift arbitrary
        let tv = head $ runTrampoline $ sample 1 arbitrary

        when "we press the power button twice"
        let tv' = pressPowerButton tv

        it "should be in the same state"
        -- lift $ quickCheck pressTwice
        assert' (tv == tv') "tv should be in the same state"
